var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { ApolloLink, Observable as LinkObservable } from 'apollo-link';
import { BatchLink } from 'apollo-link-batch';
import { print } from 'graphql/language/printer';
import { fetch, mergeHeaders, prioritize, } from 'apollo-angular-link-http-common';
var defaults = {
    batchInterval: 10,
    batchMax: 10,
    uri: 'graphql',
    method: 'POST',
};
var HttpBatchLinkHandler = /** @class */ (function (_super) {
    __extends(HttpBatchLinkHandler, _super);
    function HttpBatchLinkHandler(httpClient, options) {
        var _this = _super.call(this) || this;
        _this.httpClient = httpClient;
        _this.options = options;
        _this.batchInterval = options.batchInterval || defaults.batchInterval;
        _this.batchMax = options.batchMax || defaults.batchMax;
        var batchHandler = function (operations) {
            return new LinkObservable(function (observer) {
                var body = _this.createBody(operations);
                var headers = _this.createHeaders(operations);
                var _a = _this.createOptions(operations), method = _a.method, uri = _a.uri, withCredentials = _a.withCredentials;
                var req = {
                    method: method,
                    url: uri,
                    body: body,
                    options: {
                        withCredentials: withCredentials,
                        headers: headers,
                    },
                };
                var sub = fetch(req, _this.httpClient).subscribe({
                    next: function (result) { return observer.next(result.body); },
                    error: function (err) { return observer.error(err); },
                    complete: function () { return observer.complete(); },
                });
                return function () {
                    if (!sub.closed) {
                        sub.unsubscribe();
                    }
                };
            });
        };
        var batchKey = options.batchKey ||
            (function (operation) {
                return _this.createBatchKey(operation);
            });
        _this.batcher = new BatchLink({
            batchInterval: _this.batchInterval,
            batchMax: _this.batchMax,
            batchKey: batchKey,
            batchHandler: batchHandler,
        });
        return _this;
    }
    HttpBatchLinkHandler.prototype.createOptions = function (operations) {
        var context = operations[0].getContext();
        return {
            method: prioritize(context.method, this.options.method, defaults.method),
            uri: prioritize(context.uri, this.options.uri, defaults.uri),
            withCredentials: prioritize(context.withCredentials, this.options.withCredentials),
        };
    };
    HttpBatchLinkHandler.prototype.createBody = function (operations) {
        var _this = this;
        return operations.map(function (operation) {
            var includeExtensions = prioritize(operation.getContext().includeExtensions, _this.options.includeExtensions, false);
            var includeQuery = prioritize(operation.getContext().includeQuery, _this.options.includeQuery, true);
            var body = {
                operationName: operation.operationName,
                variables: operation.variables,
            };
            if (includeExtensions) {
                body.extensions = operation.extensions;
            }
            if (includeQuery) {
                body.query = print(operation.query);
            }
            return body;
        });
    };
    HttpBatchLinkHandler.prototype.createHeaders = function (operations) {
        return operations.reduce(function (headers, operation) {
            return mergeHeaders(headers, operation.getContext().headers);
        }, this.options.headers);
    };
    HttpBatchLinkHandler.prototype.createBatchKey = function (operation) {
        var context = operation.getContext();
        if (context.skipBatching) {
            return Math.random()
                .toString(36)
                .substr(2, 9);
        }
        var headers = context.headers &&
            context.headers.keys().map(function (k) { return context.headers.get(k); });
        var opts = JSON.stringify({
            includeQuery: context.includeQuery,
            includeExtensions: context.includeExtensions,
            headers: headers,
        });
        return prioritize(context.uri, this.options.uri) + opts;
    };
    HttpBatchLinkHandler.prototype.request = function (op) {
        return this.batcher.request(op);
    };
    return HttpBatchLinkHandler;
}(ApolloLink));
export { HttpBatchLinkHandler };
var HttpBatchLink = /** @class */ (function () {
    function HttpBatchLink(httpClient) {
        this.httpClient = httpClient;
    }
    HttpBatchLink.prototype.create = function (options) {
        return new HttpBatchLinkHandler(this.httpClient, options);
    };
    HttpBatchLink.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    HttpBatchLink.ctorParameters = function () { return [
        { type: HttpClient, },
    ]; };
    return HttpBatchLink;
}());
export { HttpBatchLink };
//# sourceMappingURL=HttpBatchLink.js.map